package com.entropy.david.Letras;

import android.annotation.TargetApi;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Typeface;
import android.os.Build;
import android.util.Log;
import android.view.MotionEvent;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.entropy.david.Medidas.Medidas;
import com.entropy.david.Movimientos.Movimiento;

//import sadf.myapplication7.app.Medidas.Medidas;
//import sadf.myapplication7.app.Movimientos.Movimiento;

/**
 * Created by david on 11/06/2014.
 */
@TargetApi(Build.VERSION_CODES.HONEYCOMB)
public class Caracter {

    //orientacion
    static final boolean ORIENTACION_HORIZONTAL = false;   //landscape --      dimension X
    static final boolean ORIENTACION_VERTICAL = true;      //portrait |        dimension Y
    private static boolean isOrientacionVertical = ORIENTACION_VERTICAL;

    //desvio
    static private Medidas medidas = Medidas.getMedidas();
    static private double DISTANCIA_DE_RANGO_MAXIMA = medidas.DISTANCIA_DE_RANGO_MAXIMA;
    static private int DESVIO_Y_VERTICAL = medidas.DESVIO_Y_VERTICAL;
    static private int DESVIO_X_VERTICAL = medidas.DESVIO_X_VERTICAL;
    static private int DESVIO_Y_HORIZONTAL = medidas.DESVIO_Y_HORIZONTAL;
    static private int DESVIO_X_HORIZONTAL = medidas.DESVIO_X_HORIZONTAL;
    static private double DISTANCIA_DE_RANGO_MAXIMA_MUY_CERCA = medidas.DISTANCIA_DE_RANGO_MAXIMA_MUY_CERCA;

    //movimiento
    private Movimiento mov;

    //objetos propios
    private TextView textView;
    private int color;
    private char valor;
    private int tipoLetra;
    private boolean estaPulsado;

    public Caracter(char valor, int col){
        this.valor = valor;
        this.color = col;
        estaPulsado = false;
    }

    public Caracter(char valor, RelativeLayout relativeLayout, final Context context, int col, float aumentoPorcentual, float i, int j, int tipoMovimiento, int tipoLetra) {
        this.color = col;
        this.valor = valor;
//        this.tipoLetra = tipoLetra;
        textView = new TextView(context);
        textView.setText(valor + "");
        textView.setTextColor(col);
        textView.setTextSize(18 * (1f + aumentoPorcentual));
        estaPulsado = false;
        setX(i);//        textView.setX(i);
        setY(j);//        textView.setY(j);

        relativeLayout.addView(textView);

        mov = Movimiento.getMovimiento(tipoMovimiento);
        mov.setCaracter(this);

        setTipoLetra(context, tipoLetra);

    }

    public Caracter(char letra, int color, int tipoLetra, Context context) {
        this.valor = letra;
        this.color = color;

        textView = new TextView(context);
        textView.setText(valor + "");
        textView.setTextColor(color);
        estaPulsado = false;

        setTipoLetra(context, tipoLetra);
    }

    public void mueve() {
        mov.mueve();
    }

    public void setTipoLetra(Context c, int tipoLetraInt) {
        this.tipoLetra = tipoLetraInt;

        Typeface tipoLetra = TypeFaces.getTipeFace(c, tipoLetraInt);

        if(mov != null &&
                mov.getTipoMovimiento() == Movimiento.MOVIMIENTO_LINEA_RECTA_REBOTE) {   //los movimientos mas dificiles
            textView.setTypeface(tipoLetra, Typeface.BOLD);          //de encontrar se ven mejor
        }
        else{
            textView.setTypeface(tipoLetra);
        }

    }

    public float getX() {
        if (isOrientacionVertical)
            return textView.getX();
        else
            return textView.getY();
    }

    public float getY() {
        if (isOrientacionVertical)
            return textView.getY();
        else
            return textView.getX();
    }

    public void setX(float x) {
        if (isOrientacionVertical)
            textView.setX(x);
        else
            textView.setY(x);
    }

    public void setY(float y) {
        if (isOrientacionVertical)
            textView.setY(y);
        else
            textView.setX(y);
    }

    public static boolean isIsOrientacionVertical() {
        return isOrientacionVertical;
    }

    public static void setIsOrientacionVertical(boolean isOrientacionVertical) {
        Caracter.isOrientacionVertical = isOrientacionVertical;
    }

    public void pulsada() {
            textView.setTextColor(Color.RED);
    }

    public void noPulsada() {
            textView.setTextColor(color);
    }

    public char getValor() {
        return valor;
    }

    public int getColor() {
        return color;
    }

    public String printInfo() {

        String s = "Valor: " + getValor();
        s += "\n color: " + color;
        s += "\n posicion x,y : " + getX() + "," + getY();
        s += "\n tipoMovimiento: " + mov.getInfo();

        return s;
    }

    public void actualiza() {
        mov.actualiza();
    }


    public boolean estaCerca(MotionEvent event) {

        float diferenciaX ;
        float diferenciaY;   //el - 120 es para calibrarlo

        if(isOrientacionVertical) {
            diferenciaX = event.getX() - this.getX() - DESVIO_X_VERTICAL;
            diferenciaY = event.getY() - this.getY() - DESVIO_Y_VERTICAL;
        }
        else{
            diferenciaX = event.getX() - this.getY() - DESVIO_X_HORIZONTAL;
            diferenciaY = event.getY() - this.getX() - DESVIO_Y_HORIZONTAL;
        }

        double distancia = diferenciaX * diferenciaX + diferenciaY * diferenciaY;
        distancia = Math.sqrt(distancia);

        if (distancia < DISTANCIA_DE_RANGO_MAXIMA) return true;
        else return false;
    }

    public boolean igual(Caracter otro){

        if (this.getValor() == otro.getValor()
                && this.getColor() == otro.getColor()) {
            return true;
        }
        else return false;

    }

    public boolean contenidoEn(Caracter[] otros){

        for(int i = 0; i< otros.length; i++) {
            if (this.getValor() == otros[i].getValor()
                    && this.getColor() == otros[i].getColor()) {
                return true;
            }
        }
        return false;

    }

    public int getTipoLetra() {
        return tipoLetra;
    }

    public Typeface getTypeFace(Context c) {
        return TypeFaces.getTipeFace(c, tipoLetra);
    }


    public void verEn(RelativeLayout rl){
        rl.addView(textView);
    }
}