package com.entropy.david.Manejadores;


import android.view.MotionEvent;

import com.entropy.david.Letras.Caracter;
import com.entropy.david.Medidas.Medidas;

import java.util.ArrayList;


/**
 * Created by david on 11/06/2014.
 */
public abstract class Manejador{

    protected static final int PASOS = 20; //anteriormente 6 //comodo en 3
    protected static final int VELOCIDAD_MAXIMA = PASOS/7+1; //anteriormente 6 //comodo en 3

    //propiedades de pantalla
    static float TAMAÑO_MAX_PANTALLA_Y = Medidas.TAMAÑO_MAX_PANTALLA_Y;//1400;
    static float TAMAÑO_MAX_PANTALLA_X = Medidas.TAMAÑO_MAX_PANTALLA_X;//1050; //900;
    static float TAMAÑO_MIN_PANTALLA_Y = Medidas.TAMAÑO_MIN_PANTALLA_Y;// 100;
    static float TAMAÑO_MIN_PANTALLA_X = Medidas.TAMAÑO_MIN_PANTALLA_X;//50;
    static float ANCHO_PANTALLA_X = TAMAÑO_MAX_PANTALLA_X - TAMAÑO_MIN_PANTALLA_X;
    static float LARGO_PANTALLA_Y = TAMAÑO_MAX_PANTALLA_Y - TAMAÑO_MIN_PANTALLA_Y;

    //orientacion
    static final boolean ORIENTACION_HORIZONTAL = false;   //landscape --      dimension X
    static final boolean ORIENTACION_VERTICAL = true;      //portrait |        dimension Y
    private static boolean isOrientacionVertical = ORIENTACION_VERTICAL;

    //propiedades de movimiento
    public static final int MANEJADOR_ESTATICO = 0;
    public static final int MANEJADOR_LASER = 1;
    public static final int MANEJADOR_GUSANO = 2;
    public static final int MANEJADOR_BLOQUE_INFINITO = 3;

    //atributos
    protected Caracter[] letras;

    protected float a;
    protected float b;
    protected ArrayList<Punto> puntos;

    public abstract int getTipoManejador();

    protected abstract void inicializacionConcreta();

    public abstract void mueve();

    public Manejador(Caracter[] letras){
        this.letras = letras;
    }


    public void pulsacion(MotionEvent event, int numPulsacion, ArrayList<Caracter>[] haSidoPulsado){

        for (int i = 0; i < letras.length; i++) {

            if (letras[i].estaCerca(event)) {

                haSidoPulsado[numPulsacion].add(letras[i]);

            }
        }
    }

    public ArrayList<Caracter> pulsacion(MotionEvent event){

        ArrayList<Caracter> pulsadas = new ArrayList<>();

        for (int i = 0; i < letras.length; i++) {

            if (letras[i].estaCerca(event)) {

                pulsadas.add(letras[i]);

            }
        }
        return pulsadas;
    }

    public void pulsaSeñalando(MotionEvent event, int numPulsacion, ArrayList<Caracter>[] haSidoPulsado) {

        for (int i = 0; i < letras.length; i++) {

            if (letras[i].estaCerca(event)) {
                letras[i].pulsada();

                haSidoPulsado[numPulsacion].add(letras[i]);

            } else letras[i].noPulsada();
        }
    }


    protected void movimiento_vertical(Caracter c) {

        if( c.getY() < TAMAÑO_MIN_PANTALLA_Y  || TAMAÑO_MAX_PANTALLA_Y < c.getY()   ) {
            b = -b;
        }
        c.setY(c.getY() + b);
    }

    protected void movimiento_horizontal(Caracter c) {

        if( c.getX() < TAMAÑO_MIN_PANTALLA_X || TAMAÑO_MAX_PANTALLA_X < c.getX())
            a = -a;

        c.setX(c.getX() + a);
    }

    protected void inicializaVelocidades(double VELOCIDAD_MAXIMA) {

        //(v)2 =   (v1)2 + (v2)2
        //(v2) = ( (v)2 - (v1)2 )2
        double v = VELOCIDAD_MAXIMA;
        double v1 = (Math.random()* VELOCIDAD_MAXIMA);
        double v2 = Math.sqrt(v*v - v1*v1);


        if(Math.random()>0.5d)
            a = (float) v1;
        else a = (float) -v1;

        if(Math.random()>0.5d)
            b = (float) v2;
        else b = (float) -v2;
    }

    protected void inicializaPuntos() {

        puntos = new ArrayList<>();

        for (int i = 0; i < letras.length-1 ; i++) {

            puntos.add(new Punto(letras[i].getX(), letras[i].getY()));

            float x1 = letras[i].getX();
            float y1 = letras[i].getY();

            float x2 = letras[i+1].getX();
            float y2 = letras[i+1].getY();

            float xDif = x2-x1;
            float yDif = y2-y1;

            //crea tantos puntos entre letras (incluye los de las propias letras) como se pida
            for (int j = 0; j < PASOS; j++) {

                puntos.add(new Punto(x1+xDif*j/PASOS, y1+yDif*j/PASOS));

            }

        }
    }

    private void situaEnRango(Caracter c) {
        if(c.getX() < TAMAÑO_MIN_PANTALLA_X) c.setX(TAMAÑO_MIN_PANTALLA_X);
        if(c.getX() > TAMAÑO_MAX_PANTALLA_X) c.setX(TAMAÑO_MAX_PANTALLA_X);
        if(c.getY() < TAMAÑO_MIN_PANTALLA_Y) c.setY(TAMAÑO_MIN_PANTALLA_Y);
        if(c.getY() > TAMAÑO_MAX_PANTALLA_Y) c.setY(TAMAÑO_MAX_PANTALLA_Y);
    }

    public static void setIsOrientacionVertical(boolean isOrientacionVertical) {
        Manejador.isOrientacionVertical = isOrientacionVertical;
    }

    public void actualiza() {
        for (int i = 0; i < letras.length; i++) {
           letras[i].actualiza();

        }
    }
}
